import oscP5.*;
import netP5.*;

OscP5 oscP5;
NetAddress myRemoteLocation;

boolean sliderLifted;
int rootCount;
int curInterval = 0;
int maxInterval = 8;

int curPage = 0;
/*
  pg |  content
  0  |  slider
  1  |  button1,2
  2  |  o (rotation)
  3  |  ^ (proximity)
*/

PFont font;

int swidth = 400; int sheight = 400;
void setup() {
  colorMode(HSB, 100);
  size(swidth, sheight);
  
  oscP5 = new OscP5(this, 8000);
  
  /* myRemoteLocation is a NetAddress. a NetAddress takes 2 parameters,
   * an ip address and a port number. myRemoteLocation is used as parameter in
   * oscP5.send() when sending osc packets to another computer, device, 
   * application. usage see below. for testing purposes the listening port
   * and the port of the remote location address are the same, hence you will
   * send messages back to this sketch.
   */
  myRemoteLocation = new NetAddress("127.0.0.1",3000);
  
  rootCount = 0;
  
  font = loadFont("Serif-48.vlw");
  textFont(font);
}


void draw() {
  background(45, 80, (curInterval*1.0/maxInterval)*60+40);
  text(curInterval,swidth/2-10,sheight/2);
}


void oscEvent(OscMessage theOscMessage) {
  
  // PRINTS.
  String addrPattern = theOscMessage.addrPattern();
  String typeTag = theOscMessage.typetag();
  if (!addrPattern.equals("/accxyz")) {
  print("OSC Message: ");
  print("     AddrPattern: "+addrPattern);
  print("     TypeTag: "+typeTag);
  println();
  }
  
  //PRE-SET CURRENT PAGE TO FIRST ADDRESS. //(still won't work for page3,4, as they're gestural, not touch.)
  String[] addresses = addrPattern.split("/");
  String firstAddress = addresses[1];
  try{
  curPage = Integer.parseInt(firstAddress);
  } catch(Exception e) {};
  /**/
  
  // SLIDER.
  if (curPage==0) {
    if (addrPattern.equals("/0/slider")) {
    if (typeTag.equals("f")) {
      float fvalue = theOscMessage.get(0).floatValue();
      curInterval = (int)(fvalue*maxInterval+0.5);
    }
    }
  }
  
  // BUTTONS.
  if (curPage==1) {
    if (addrPattern.equals("/1/button1/z")) {
      float fvalue = theOscMessage.get(0).floatValue();
      if (fvalue==1) {
        curInterval +=1;
      }
    }
    if (addrPattern.equals("/1/button2/z")) {
      float fvalue = theOscMessage.get(0).floatValue();
      if (fvalue==1) {
        curInterval -=1;
      }
    }
  }
  
  // ROTATION.
  if (curPage==2) {
    if (addrPattern.equals("/accxyz")) {
      this.handleAccelerationInput(theOscMessage);
    }
  }
  
  // PROXIMITY.
  if (curPage==3) {
    if (addrPattern.equals("/proximity")) {
      float fvalue = theOscMessage.get(0).floatValue();
      curInterval = (int) map(fvalue,0.25,0.8,maxInterval,0);
    }
  }
  
  // INTERVAL BOUNDS.
  if (curInterval<0) curInterval = 0; if(curInterval>maxInterval) curInterval = maxInterval;
  
  // CHANGE PAGES.
  if (typeTag.equals("")) {
    String newPage = addrPattern.replace("/","");
    curPage = Integer.parseInt(newPage);
  }
  
  // OSC Messages.
  
  OscMessage myMessage;
  
  // FOCUS. FOWARDING.
  if (addrPattern.equals("/focus")) {
    float fvalue = theOscMessage.get(0).floatValue();
     // SEND OSC MESSAGE.
    myMessage = new OscMessage("/focus");
    myMessage.add(fvalue);
    oscP5.send(myMessage,myRemoteLocation);
  }
  
  // SEND OSC MESSAGE.
  myMessage = new OscMessage("/level");
  myMessage.add(curInterval);
  oscP5.send(myMessage,myRemoteLocation);
  
}

int acc_maxIntervalDifference = 1;//0;//1;
int acc_curIntervalSettlingTime = 0;
int acc_minIntervalSettlingTime = 5;//20;//2;
int acc_curSettlingInterval = 0;

void handleAccelerationInput(OscMessage theOscMessage) {
  if (theOscMessage.checkAddrPattern("/accxyz") == true) {
        float fz = theOscMessage.get(2).floatValue(); //using z-rotation only.
        fz=(fz+1.0f)/2.0;
        fz = (int)(fz*maxInterval+0.5f)/(maxInterval*1.0f);
        
        int angleInterval = (int)(fz*maxInterval);
        if (abs(angleInterval-curInterval)>acc_maxIntervalDifference) {
          //CHANGE INTERVAL. //IF INTERVAL DIFFERENCE TOO GREAT! //(Note, an extra timer could also be added to this for glitch points)
          curInterval = angleInterval;
          acc_curIntervalSettlingTime = 0;
        } 
        else {
          //if angleInterval is different to curInterval
          if (angleInterval!=curInterval) {

            //if angleInterval is same as curSettlingInterval - add.
            //if different then reset curSettlingInterval and curIntervalSettlingTime.
            if (angleInterval==acc_curSettlingInterval) {
              
              acc_curIntervalSettlingTime++;
              if (acc_curIntervalSettlingTime>acc_minIntervalSettlingTime) {
                //CHANGE INTERVAL. //a small interval. //IF HELD.
                curInterval = angleInterval;
                acc_curIntervalSettlingTime = 0;
              }
            } else {
              acc_curIntervalSettlingTime = 0;
              acc_curSettlingInterval = angleInterval;
            }
          }
        }
      }
}

