// Handles the visualisation, and translating the OSC data (and accelerometre data) from the iPhone to useful interactions.

import oscP5.*;
import netP5.*;

import maxlink.*;

MaxLink link = new MaxLink(this, "vis147finalprocessing");

public float peakValue = 0;


import processing.serial.*;
Serial port;

OscP5 oscP5;
NetAddress myRemoteLocation;

boolean sliderLifted;
int rootCount;
int curInterval = 0;
int maxInterval = 100;

int curPage = 0;
/*
  pg |  content
 0  |  slider
 1  |  button1,2
 2  |  o (rotation)
 3  |  ^ (proximity)
 */

PFont font;

int swidth = 800; 
int sheight = 600;
void setup() {
  colorMode(HSB, 100);
  size(swidth, sheight);

  oscP5 = new OscP5(this, 8000);
  
  myRemoteLocation = new NetAddress("127.0.0.1", 3000);

  rootCount = 0;

  font = loadFont("Serif-48.vlw");
  textFont(font);


  link.declareInlet("peakValue");


  colorMode(HSB, 100);
  background(0, 0, 0);
  
  
  println("Available serial ports:");
  println(Serial.list());

  port = new Serial(this, Serial.list()[0], 9600);  
}

float translatedSpeed = 1.0;
float value = 0.0;

void draw() {
  //background(45, 80, (curInterval*1.0/maxInterval)*60+40);
  //text(curInterval, swidth/2-10, sheight/2);

  translatedSpeed = map(curInterval, 0, maxInterval, -2.0, 2.0);
  
  //------
  //demo code (osc not working)
  //translatedSpeed = map(mouseX-swidth/2, 0 , swidth/2 , 0 , 2);
  //println(translatedSpeed);
  
  //------
  
  link.output(translatedSpeed);
  
  value = map(peakValue,0.0,0.5,0.0,1.0); // <------------------------------------------- EDIT HERE BASED ON ENVIRO.

  drawVis();
  
  commandArduino();
}

void commandArduino() {
  
  int a = (int) map(translatedSpeed,-2.0,2.0,0,254);
  
  int b = (int) map( value, 0.0, 1.0, 0, 254) ;
  
  
  port.write('A'); //breakpoint
  port.write( a  );  //time ∂. influences the leds.
  port.write( b ); //peak. influences led & motor.
  
}

int rootFrame = 0;
float rootAngle = 0;
float rootAngleChange = 0.02;

float hueCur = 0;
float hueChangeMax = 0.5;
PImage prvFrameImg;

void drawVis() {

  //----------------------------------------------------------------------------
  //fading bg
  pushMatrix();
  translate(swidth/2, sheight/2);
  //scale(2.003f);
  scale(1.05f);

  if (prvFrameImg!=null) { 
    image(prvFrameImg, -swidth/2, -sheight/2);
  }
  noStroke();
  fill(0, 1.25f);
  rect(-swidth/2, -sheight/2, swidth, sheight);
  popMatrix();

  //----------------------------------------------------------------------------
  //draw saved image
  rootAngle += rootAngleChange;

  pushMatrix();
  translate(swidth/2, sheight/2);
  rotate(rootAngle);
  hueCur = (hueCur+hueChangeMax*noise(1))%100;
  fill(hueCur, 100, 100);
  translate(0, 0);
  float prvY = -sheight/2*0.9 * value;//prvDifferentSerialValue;
  float curY = -sheight/2*0.9 * value;//serialValue;

  prvY = curY;

  int r = 5;
  ellipse(0, curY, r, r);
  strokeWeight(1);
  stroke(hueCur, 100, 50);
  //line(0, prvY, 0, curY);
  if (rootFrame%200==0) {
    line(0, 0, 0, curY);
  }


  popMatrix();

  prvFrameImg = createImage(swidth, sheight, HSB);
  prvFrameImg = get();

  //----------------------------------------------------------------------------
  //After prvFrameImg Drawing / Overlay.
  pushMatrix();
  translate(swidth/2, sheight/2);
  rotate(rootAngle);

  strokeWeight(0.5f);
  line(0, 0, 0, curY);
  noFill();
  strokeWeight(2);

  curY = -sheight/2*0.9 * value;//smoothedSerialValue;
  ellipse(0, 0, curY*2, curY*2);


  popMatrix();
}


void oscEvent(OscMessage theOscMessage) {

  // PRINTS.
  String addrPattern = theOscMessage.addrPattern();
  String typeTag = theOscMessage.typetag();
  if (!addrPattern.equals("/accxyz")) {
    print("OSC Message: ");
    print("     AddrPattern: "+addrPattern);
    print("     TypeTag: "+typeTag);
    println();
  }

  //PRE-SET CURRENT PAGE TO FIRST ADDRESS. //(still won't work for page3,4, as they're gestural, not touch.)
  String[] addresses = addrPattern.split("/");
  String firstAddress = addresses[1];
  try {
    curPage = Integer.parseInt(firstAddress);
  } 
  catch(Exception e) {
  };
  /**/

  // SLIDER.
  if (curPage==0) {
    if (addrPattern.equals("/0/slider")) {
      if (typeTag.equals("f")) {
        float fvalue = theOscMessage.get(0).floatValue();
        curInterval = (int)(fvalue*maxInterval+0.5);
      }
    }
  }

  // BUTTONS.
  if (curPage==1) {
    if (addrPattern.equals("/1/button1/z")) {
      float fvalue = theOscMessage.get(0).floatValue();
      if (fvalue==1) {
        curInterval +=1;
      }
    }
    if (addrPattern.equals("/1/button2/z")) {
      float fvalue = theOscMessage.get(0).floatValue();
      if (fvalue==1) {
        curInterval -=1;
      }
    }
  }

  // ROTATION.
  if (curPage==2) {
    if (addrPattern.equals("/accxyz")) {
      this.handleAccelerationInput(theOscMessage);
    }
  }

  // PROXIMITY.
  if (curPage==3) {
    if (addrPattern.equals("/proximity")) {
      float fvalue = theOscMessage.get(0).floatValue();
      curInterval = (int) map(fvalue, 0.25, 0.8, maxInterval, 0);
    }
  }

  // INTERVAL BOUNDS.
  if (curInterval<0) curInterval = 0; 
  if (curInterval>maxInterval) curInterval = maxInterval;

  // CHANGE PAGES.
  if (typeTag.equals("")) {
    String newPage = addrPattern.replace("/", "");
    curPage = Integer.parseInt(newPage);
  }

  // OSC Messages.

  OscMessage myMessage;

  // FOCUS. FOWARDING.
  if (addrPattern.equals("/focus")) {
    float fvalue = theOscMessage.get(0).floatValue();
    // SEND OSC MESSAGE.
    myMessage = new OscMessage("/focus");
    myMessage.add(fvalue);
    oscP5.send(myMessage, myRemoteLocation);
  }

  // SEND OSC MESSAGE.
  myMessage = new OscMessage("/level");
  myMessage.add(curInterval);
  oscP5.send(myMessage, myRemoteLocation);
}

int acc_maxIntervalDifference = 1;//0;//1;
int acc_curIntervalSettlingTime = 0;
int acc_minIntervalSettlingTime = 5;//20;//2;
int acc_curSettlingInterval = 0;

void handleAccelerationInput(OscMessage theOscMessage) {
  if (theOscMessage.checkAddrPattern("/accxyz") == true) {
    float fz = theOscMessage.get(2).floatValue(); //using z-rotation only.
    fz=(fz+1.0f)/2.0;
    fz = (int)(fz*maxInterval+0.5f)/(maxInterval*1.0f);

    int angleInterval = (int)(fz*maxInterval);
    if (abs(angleInterval-curInterval)>acc_maxIntervalDifference) {
      //CHANGE INTERVAL. //IF INTERVAL DIFFERENCE TOO GREAT! //(Note, an extra timer could also be added to this for glitch points)
      curInterval = angleInterval;
      acc_curIntervalSettlingTime = 0;
    } 
    else {
      //if angleInterval is different to curInterval
      if (angleInterval!=curInterval) {

        //if angleInterval is same as curSettlingInterval - add.
        //if different then reset curSettlingInterval and curIntervalSettlingTime.
        if (angleInterval==acc_curSettlingInterval) {

          acc_curIntervalSettlingTime++;
          if (acc_curIntervalSettlingTime>acc_minIntervalSettlingTime) {
            //CHANGE INTERVAL. //a small interval. //IF HELD.
            curInterval = angleInterval;
            acc_curIntervalSettlingTime = 0;
          }
        } 
        else {
          acc_curIntervalSettlingTime = 0;
          acc_curSettlingInterval = angleInterval;
        }
      }
    }
  }
}

